@@ -5,7 +5,7 @@ public:
   deadcall(char *id, char * reason);
   ~deadcall();
 
-  virtual bool process_incoming(char * msg);
+  virtual bool process_incoming(char * msg, struct sockaddr_storage *);
   virtual bool  process_twinSippCom(char * msg);
 
   virtual bool run();
@@ -13,12 +13,10 @@ public:
   /* When should this call wake up? */
   virtual unsigned int wake();
 
-  virtual void abort();
-
   /* Dump call info to error log. */
   virtual void dump();
 
 protected:
-  int expiration;
+  unsigned long expiration;
   char *reason;
 };
