@@ -34,7 +34,7 @@ public:
   listener(char *id, bool listening);
   virtual ~listener();
   char *getId();
-  virtual bool process_incoming(char * msg) = 0;
+  virtual bool process_incoming(char * msg, struct sockaddr_storage *src) = 0;
   virtual bool process_twinSippCom(char * msg) = 0;
 
 protected:
